/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2012 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::alphaWeightedMixedFvPatchScalarField

Group
    grpGenericBoundaryConditions

Description
    This boundary condition provides a phase-based hydrostatic pressure
    condition, calculated as:

        \f[
        \f]

    where
    \vartable
        p_{hyd} | hyrostatic pressure [Pa]
        p_{ref} | reference pressure [Pa]
    \endtable

    The values are assigned according to the phase-fraction field:
    - 1: apply \$fp_{hyd}\$f
    - 0: apply a zero-gradient condition

    \heading Patch usage

    \table
        Property     | Description             | Required    | Default value
        phaseName    | phase field name        | no          | alpha
        pRefValue    | reference pressure [Pa] | yes         |
    \endtable

    Example of the boundary condition specification:
    \verbatim
    myPatch
    {
        type            phaseHydrostaticPressure;
        phaseName       alpha1;
        pRefValue       1e5;
        value           uniform 0; // optional initial value
    }
    \endverbatim

SeeAlso
    Foam::mixedFvPatchScalarField

SourceFiles
    alphaWeightedMixedFvPatchScalarField.C

\*---------------------------------------------------------------------------*/

#ifndef alphaWeightedMixedFvPatchScalarField_H
#define alphaWeightedMixedFvPatchScalarField_H

#include "mixedFvPatchFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
         Class alphaWeightedMixedFvPatchScalarField Declaration
\*---------------------------------------------------------------------------*/

class alphaWeightedMixedFvPatchScalarField
:
    public mixedFvPatchScalarField
{

protected:

    // Protected data

        //- Name of phase-fraction field
        word phaseName_;

        //- Reference pressure
        scalar pRefValue_;


public:

    //- Runtime type information
    TypeName("alphaWeighted");


    // Constructors

        //- Construct from patch and internal field
        alphaWeightedMixedFvPatchScalarField
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct from patch, internal field and dictionary
        alphaWeightedMixedFvPatchScalarField
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const dictionary&
        );

        //- Construct by mapping given
        //  alphaWeightedMixedFvPatchScalarField onto a new patch
        alphaWeightedMixedFvPatchScalarField
        (
            const alphaWeightedMixedFvPatchScalarField&,
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const fvPatchFieldMapper&
        );

        //- Construct as copy
        alphaWeightedMixedFvPatchScalarField
        (
            const alphaWeightedMixedFvPatchScalarField&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchScalarField> clone() const
        {
            return tmp<fvPatchScalarField >
            (
                new alphaWeightedMixedFvPatchScalarField(*this)
            );
        }

        //- Construct as copy setting internal field reference
        alphaWeightedMixedFvPatchScalarField
        (
            const alphaWeightedMixedFvPatchScalarField&,
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchScalarField> clone
        (
            const DimensionedField<scalar, volMesh>& iF
        ) const
        {
            return tmp<fvPatchScalarField>
            (
                new alphaWeightedMixedFvPatchScalarField(*this, iF)
            );
        }


    // Member functions

        // Access

            //- Return the phaseName
            const word& phaseName() const
            {
                return phaseName_;
            }

            //- Return reference to the phaseName to allow adjustment
            word& phaseName()
            {
                return phaseName_;
            }

            //- Return the reference pressure
            scalar pRefValue() const
            {
                return pRefValue_;
            }

            //- Return reference to the reference pressure to allow adjustment
            scalar& pRefValue()
            {
                return pRefValue_;
            }


        //- Update the coefficients associated with the patch field
        virtual void updateCoeffs();

        //- Write
        virtual void write(Ostream&) const;


    // Member operators

        virtual void operator=(const fvPatchScalarField& pvf);
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
