/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2012 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "freeSurfaceOutletMixedFvPatchScalarField.H"
#include "addToRunTimeSelectionTable.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "uniformDimensionedFields.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::freeSurfaceOutletMixedFvPatchScalarField::
freeSurfaceOutletMixedFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF
)
:
    mixedFvPatchScalarField(p, iF),
    rho_(0.0),
    steelWaterLevel_(vector::zero),
    bottomLevel_(vector::zero),
    pRefValue_(0.0)
{
    this->refValue() = 0.0;
    this->refGrad() = 0.0;
    this->valueFraction() = 0.0;
}


Foam::freeSurfaceOutletMixedFvPatchScalarField::
freeSurfaceOutletMixedFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const dictionary& dict
)
:
    mixedFvPatchScalarField(p, iF),
    rho_(readScalar(dict.lookup("rho"))),
    steelWaterLevel_(dict.lookup("steelWaterLevel")),
    bottomLevel_(dict.lookup("bottomLevel"))
{
//    const uniformDimensionedVectorField& g =
//        db().lookupObject<uniformDimensionedVectorField>("g");

//    pRefValue_ = -rho_*(g.value() & steelWaterLevel_);
    pRefValue_ = 0.0;

    this->refValue() = pRefValue_;

    if (dict.found("value"))
    {
        fvPatchScalarField::operator=
        (
            scalarField("value", dict, p.size())
        );
    }
    else
    {
        fvPatchScalarField::operator=(this->refValue());
    }

    this->refGrad() = 0.0;
    this->valueFraction() = 0.0;
}


Foam::freeSurfaceOutletMixedFvPatchScalarField::
freeSurfaceOutletMixedFvPatchScalarField
(
    const freeSurfaceOutletMixedFvPatchScalarField& ptf,
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    mixedFvPatchScalarField(ptf, p, iF, mapper),
    rho_(ptf.rho_),
    steelWaterLevel_(ptf.steelWaterLevel_),
    bottomLevel_(ptf.bottomLevel_)
{}


Foam::freeSurfaceOutletMixedFvPatchScalarField::
freeSurfaceOutletMixedFvPatchScalarField
(
    const freeSurfaceOutletMixedFvPatchScalarField& ptf
)
:
    mixedFvPatchScalarField(ptf)
{}


Foam::freeSurfaceOutletMixedFvPatchScalarField::
freeSurfaceOutletMixedFvPatchScalarField
(
    const freeSurfaceOutletMixedFvPatchScalarField& ptf,
    const DimensionedField<scalar, volMesh>& iF
)
:
    mixedFvPatchScalarField(ptf, iF),
    rho_(ptf.rho_),
    steelWaterLevel_(ptf.steelWaterLevel_),
    bottomLevel_(ptf.bottomLevel_)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::freeSurfaceOutletMixedFvPatchScalarField::updateCoeffs()
{
    if (this->updated())
    {
        return;
    }

    const uniformDimensionedVectorField& g =
        db().lookupObject<uniformDimensionedVectorField>("g");

    pRefValue_ = -rho_ * (g.value() & steelWaterLevel_);
    refValue() = pRefValue_;

    scalar depth = -g.value() & (steelWaterLevel_ - bottomLevel_) * 0.8 /mag(g.value());

    valueFraction()  = 
        1.0 - min
        (
            max
            (
                mag( g.value() & (patch().Cf()-bottomLevel_) )/mag(g.value())/depth
                ,scalar(0)
            )
            ,scalar(1)
        );

    mixedFvPatchScalarField::updateCoeffs();
}


void Foam::freeSurfaceOutletMixedFvPatchScalarField::write(Ostream& os) const
{
    fvPatchScalarField::write(os);
    os.writeKeyword("rho") << rho_ << token::END_STATEMENT << nl;
    os.writeKeyword("steelWaterLevel") << steelWaterLevel_ << token::END_STATEMENT << nl;
    os.writeKeyword("bottomLevel") << bottomLevel_ << token::END_STATEMENT << nl;
    writeEntry("value", os);
}


// * * * * * * * * * * * * * * * Member Operators  * * * * * * * * * * * * * //

void Foam::freeSurfaceOutletMixedFvPatchScalarField::operator=
(
    const fvPatchScalarField& ptf
)
{
    fvPatchScalarField::operator=
    (
        valueFraction()*refValue()
        + (1 - valueFraction())*ptf
    );
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
    makePatchTypeField
    (
        fvPatchScalarField,
        freeSurfaceOutletMixedFvPatchScalarField
    );
}

// ************************************************************************* //
